@@ -1,7 +1,7 @@
 //! Like `wheel.rs`, but for installing wheels that have already been unzipped, rather than
 //! reading from a zip file.
 
-use std::path::{Path, PathBuf};
+use std::path::Path;
 use std::str::FromStr;
 use std::time::SystemTime;
 
@@ -16,6 +16,7 @@ use distribution_filename::WheelFilename;
 use pep440_rs::Version;
 use pypi_types::DirectUrl;
 use uv_normalize::PackageName;
+use uv_warnings::warn_user_once;
 
 use crate::script::{scripts_from_ini, Script};
 use crate::wheel::{
@@ -143,24 +144,6 @@ pub fn install_wheel(
     Ok(())
 }
 
-/// Determine the absolute path to an entrypoint script.
-pub fn entrypoint_path(entrypoint: &Script, layout: &Layout) -> PathBuf {
-    if cfg!(windows) {
-        // On windows we actually build an .exe wrapper
-        let script_name = entrypoint
-            .name
-            // FIXME: What are the in-reality rules here for names?
-            .strip_suffix(".py")
-            .unwrap_or(&entrypoint.name)
-            .to_string()
-            + ".exe";
-
-        layout.scheme.scripts.join(script_name)
-    } else {
-        layout.scheme.scripts.join(&entrypoint.name)
-    }
-}
-
 /// Find the `dist-info` directory in an unzipped wheel.
 ///
 /// See: <https://github.com/PyO3/python-pkginfo-rs>
@@ -373,7 +356,8 @@ fn clone_recursive(
                             debug!(
                                 "Failed to clone `{}` to temporary location `{}`, attempting to copy files as a fallback",
                                 from.display(),
-                                tempfile.display());
+                                tempfile.display(),
+                            );
                             *attempt = Attempt::UseCopyFallback;
                             fs::copy(&from, &to)?;
                         }
@@ -419,6 +403,7 @@ fn clone_recursive(
             } else {
                 fs::copy(&from, &to)?;
             }
+            warn_user_once!("Failed to clone files; falling back to full copy. This may lead to degraded performance. If this is intentional, use `--link-mode=copy` to suppress this warning.\n\nhint: If the cache and target directories are on different filesystems, reflinking may not be supported.");
         }
     }
 
@@ -542,6 +527,7 @@ fn hardlink_wheel_files(
             }
             Attempt::UseCopyFallback => {
                 fs::copy(path, &out_path)?;
+                warn_user_once!("Failed to hardlink files; falling back to full copy. This may lead to degraded performance. If this is intentional, use `--link-mode=copy` to suppress this warning.\n\nhint: If the cache and target directories are on different filesystems, hardlinking may not be supported.");
             }
         }
 
