@@ -17,7 +17,7 @@ use distribution_filename::WheelFilename;
 use distribution_types::{
     BuiltDist, DirectUrlBuiltDist, DirectUrlSourceDist, DirectorySourceDist, Dist, FileLocation,
     GitSourceDist, IndexUrl, PathBuiltDist, PathSourceDist, RegistryBuiltDist, RegistryBuiltWheel,
-    RegistrySourceDist, RemoteSource, Resolution, ResolvedDist, ToUrlError,
+    RegistrySourceDist, RemoteSource, Resolution, ResolvedDist, ToUrlError, UrlString,
 };
 use pep440_rs::{Version, VersionSpecifiers};
 use pep508_rs::{
@@ -38,7 +38,7 @@ use crate::{RequiresPython, ResolutionGraph};
 /// The current version of the lock file format.
 const VERSION: u32 = 1;
 
-#[derive(Clone, Debug, serde::Deserialize)]
+#[derive(Clone, Debug, serde::Deserialize, PartialEq, Eq)]
 #[serde(try_from = "LockWire")]
 pub struct Lock {
     version: u32,
@@ -324,6 +324,11 @@ impl Lock {
         &self.distributions
     }
 
+    /// Returns the owned [`Distribution`] entries in this lock.
+    pub fn into_distributions(self) -> Vec<Distribution> {
+        self.distributions
+    }
+
     /// Returns the supported Python version range for the lockfile, if present.
     pub fn requires_python(&self) -> Option<&RequiresPython> {
         self.requires_python.as_ref()
@@ -509,7 +514,7 @@ impl TryFrom<LockWire> for Lock {
     }
 }
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Distribution {
     pub(crate) id: DistributionId,
     sdist: Option<SourceDist>,
@@ -711,7 +716,7 @@ impl Distribution {
                         requires_python: None,
                         size: sdist.size(),
                         upload_time_utc_ms: None,
-                        url: FileLocation::AbsoluteUrl(file_url.to_string()),
+                        url: FileLocation::AbsoluteUrl(file_url.clone().into()),
                         yanked: None,
                     });
                     let index = IndexUrl::Url(VerbatimUrl::from_url(url.clone()));
@@ -817,10 +822,6 @@ impl Distribution {
 
         self.id.to_toml(None, &mut table);
 
-        if let Some(ref sdist) = self.sdist {
-            table.insert("sdist", value(sdist.to_toml()?));
-        }
-
         if !self.dependencies.is_empty() {
             let deps = each_element_on_its_line_array(
                 self.dependencies
@@ -854,6 +855,10 @@ impl Distribution {
             table.insert("dev-dependencies", Item::Table(dev_dependencies));
         }
 
+        if let Some(ref sdist) = self.sdist {
+            table.insert("sdist", value(sdist.to_toml()?));
+        }
+
         if !self.wheels.is_empty() {
             let wheels = each_element_on_its_line_array(
                 self.wheels
@@ -1316,12 +1321,15 @@ enum SourceWire {
         subdirectory: Option<String>,
     },
     Path {
+        #[serde(deserialize_with = "deserialize_path_with_dot")]
         path: PathBuf,
     },
     Directory {
+        #[serde(deserialize_with = "deserialize_path_with_dot")]
         directory: PathBuf,
     },
     Editable {
+        #[serde(deserialize_with = "deserialize_path_with_dot")]
         editable: PathBuf,
     },
 }
@@ -1425,7 +1433,7 @@ enum GitSourceKind {
 }
 
 /// Inspired by: <https://discuss.python.org/t/lock-files-again-but-this-time-w-sdists/46593>
-#[derive(Clone, Debug, serde::Deserialize)]
+#[derive(Clone, Debug, serde::Deserialize, PartialEq, Eq)]
 struct SourceDistMetadata {
     /// A hash of the source distribution.
     hash: Hash,
@@ -1439,7 +1447,7 @@ struct SourceDistMetadata {
 /// locked against was found. The location does not need to exist in the
 /// future, so this should be treated as only a hint to where to look
 /// and/or recording where the source dist file originally came from.
-#[derive(Clone, Debug, serde::Deserialize)]
+#[derive(Clone, Debug, serde::Deserialize, PartialEq, Eq)]
 #[serde(untagged)]
 enum SourceDist {
     Url {
@@ -1690,14 +1698,14 @@ fn locked_git_url(git_dist: &GitSourceDist) -> Url {
 }
 
 /// Inspired by: <https://discuss.python.org/t/lock-files-again-but-this-time-w-sdists/46593>
-#[derive(Clone, Debug, serde::Deserialize)]
+#[derive(Clone, Debug, serde::Deserialize, PartialEq, Eq)]
 #[serde(try_from = "WheelWire")]
 struct Wheel {
     /// A URL or file path (via `file://`) where the wheel that was locked
     /// against was found. The location does not need to exist in the future,
     /// so this should be treated as only a hint to where to look and/or
     /// recording where the wheel file originally came from.
-    url: Url,
+    url: UrlString,
     /// A hash of the built distribution.
     ///
     /// This is only present for wheels that come from registries and direct
@@ -1765,7 +1773,7 @@ impl Wheel {
         let url = wheel
             .file
             .url
-            .to_url()
+            .to_url_string()
             .map_err(LockErrorKind::InvalidFileUrl)
             .map_err(LockError::from)?;
         let hash = wheel.file.hashes.first().cloned().map(Hash::from);
@@ -1780,7 +1788,7 @@ impl Wheel {
 
     fn from_direct_dist(direct_dist: &DirectUrlBuiltDist, hashes: &[HashDigest]) -> Wheel {
         Wheel {
-            url: direct_dist.url.to_url(),
+            url: direct_dist.url.to_url().into(),
             hash: hashes.first().cloned().map(Hash::from),
             size: None,
             filename: direct_dist.filename.clone(),
@@ -1789,7 +1797,7 @@ impl Wheel {
 
     fn from_path_dist(path_dist: &PathBuiltDist, hashes: &[HashDigest]) -> Wheel {
         Wheel {
-            url: path_dist.url.to_url(),
+            url: path_dist.url.to_url().into(),
             hash: hashes.first().cloned().map(Hash::from),
             size: None,
             filename: path_dist.filename.clone(),
@@ -1805,7 +1813,7 @@ impl Wheel {
             requires_python: None,
             size: self.size,
             upload_time_utc_ms: None,
-            url: FileLocation::AbsoluteUrl(self.url.to_string()),
+            url: FileLocation::AbsoluteUrl(self.url.clone()),
             yanked: None,
         });
         let index = IndexUrl::Url(VerbatimUrl::from_url(url.clone()));
@@ -1864,7 +1872,7 @@ impl TryFrom<WheelWire> for Wheel {
             .map_err(|err| format!("failed to parse `{filename}` as wheel filename: {err}"))?;
 
         Ok(Wheel {
-            url: wire.url,
+            url: wire.url.into(),
             hash: wire.hash,
             size: wire.size,
             filename,
@@ -1887,7 +1895,7 @@ impl Dependency {
     ) -> Dependency {
         let distribution_id = DistributionId::from_annotated_dist(annotated_dist);
         let extra = annotated_dist.extra.clone();
-        let marker = marker.cloned().and_then(crate::marker::normalize);
+        let marker = marker.cloned();
         Dependency {
             distribution_id,
             extra,
@@ -2042,7 +2050,7 @@ impl From<Dependency> for DependencyWire {
 ///
 /// A hash is encoded as a single TOML string in the format
 /// `{algorithm}:{digest}`.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 struct Hash(HashDigest);
 
 impl From<HashDigest> for Hash {
