@@ -23,16 +23,17 @@ use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
 use uv_git::GitResolver;
 use uv_normalize::PackageName;
+use uv_python::{
+    EnvironmentPreference, PythonEnvironment, PythonInstallation, PythonPreference, PythonRequest,
+    PythonVersion, VersionRequest,
+};
 use uv_requirements::{
     upgrade::read_requirements_txt, RequirementsSource, RequirementsSpecification,
 };
 use uv_resolver::{
     AnnotationStyle, DependencyMode, DisplayResolutionGraph, ExcludeNewer, FlatIndex,
-    InMemoryIndex, OptionsBuilder, PreReleaseMode, PythonRequirement, ResolutionMode,
-};
-use uv_toolchain::{
-    EnvironmentPreference, PythonEnvironment, PythonVersion, Toolchain, ToolchainPreference,
-    ToolchainRequest, VersionRequest,
+    InMemoryIndex, OptionsBuilder, PreReleaseMode, PythonRequirement, RequiresPython,
+    ResolutionMode,
 };
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 use uv_warnings::warn_user;
@@ -42,7 +43,7 @@ use crate::commands::ExitStatus;
 use crate::printer::Printer;
 
 /// Resolve a set of requirements into a set of pinned versions.
-#[allow(clippy::too_many_arguments, clippy::fn_params_excessive_bools)]
+#[allow(clippy::fn_params_excessive_bools)]
 pub(crate) async fn pip_compile(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
@@ -82,7 +83,7 @@ pub(crate) async fn pip_compile(
     link_mode: LinkMode,
     python: Option<String>,
     system: bool,
-    toolchain_preference: ToolchainPreference,
+    python_preference: PythonPreference,
     concurrency: Concurrency,
     native_tls: bool,
     quiet: bool,
@@ -158,18 +159,18 @@ pub(crate) async fn pip_compile(
     // Find an interpreter to use for building distributions
     let environments = EnvironmentPreference::from_system_flag(system, false);
     let interpreter = if let Some(python) = python.as_ref() {
-        let request = ToolchainRequest::parse(python);
-        Toolchain::find(&request, environments, toolchain_preference, &cache)
+        let request = PythonRequest::parse(python);
+        PythonInstallation::find(&request, environments, python_preference, &cache)
     } else {
         // TODO(zanieb): The split here hints at a problem with the abstraction; we should be able to use
-        // `Toolchain::find(...)` here.
+        // `PythonInstallation::find(...)` here.
         let request = if let Some(version) = python_version.as_ref() {
             // TODO(zanieb): We should consolidate `VersionRequest` and `PythonVersion`
-            ToolchainRequest::Version(VersionRequest::from(version))
+            PythonRequest::Version(VersionRequest::from(version))
         } else {
-            ToolchainRequest::default()
+            PythonRequest::default()
         };
-        Toolchain::find_best(&request, environments, toolchain_preference, &cache)
+        PythonInstallation::find_best(&request, environments, python_preference, &cache)
     }?
     .into_interpreter();
 
@@ -212,7 +213,16 @@ pub(crate) async fn pip_compile(
     };
 
     // Determine the Python requirement, if the user requested a specific version.
-    let python_requirement = if let Some(python_version) = python_version.as_ref() {
+    let python_requirement = if universal {
+        let requires_python = RequiresPython::greater_than_equal_version(
+            if let Some(python_version) = python_version.as_ref() {
+                &python_version.version
+            } else {
+                interpreter.python_version()
+            },
+        );
+        PythonRequirement::from_requires_python(&interpreter, &requires_python)
+    } else if let Some(python_version) = python_version.as_ref() {
         PythonRequirement::from_python_version(&interpreter, python_version)
     } else {
         PythonRequirement::from_interpreter(&interpreter)
@@ -247,12 +257,11 @@ pub(crate) async fn pip_compile(
     }
 
     // Initialize the registry client.
-    let client = RegistryClientBuilder::new(cache.clone())
-        .native_tls(native_tls)
-        .connectivity(connectivity)
+
+    let client = RegistryClientBuilder::from(client_builder)
+        .cache(cache.clone())
         .index_urls(index_locations.index_urls())
         .index_strategy(index_strategy)
-        .keyring(keyring_provider)
         .markers(interpreter.markers())
         .platform(interpreter.platform())
         .build();
@@ -402,7 +411,7 @@ pub(crate) async fn pip_compile(
     // If necessary, include the `--find-links` locations.
     if include_find_links {
         for flat_index in index_locations.flat_index() {
-            writeln!(writer, "--find-links {flat_index}")?;
+            writeln!(writer, "--find-links {}", flat_index.verbatim())?;
             wrote_preamble = true;
         }
     }
