@@ -29,6 +29,7 @@ use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
 use uv_installer::{Plan, Planner, Preparer, SitePackages};
 use uv_normalize::{GroupName, PackageName};
+use uv_python::PythonEnvironment;
 use uv_requirements::{
     LookaheadResolver, NamedRequirementsResolver, RequirementsSource, RequirementsSpecification,
     SourceTreeResolver,
@@ -37,7 +38,6 @@ use uv_resolver::{
     DependencyMode, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options, Preference,
     Preferences, PythonRequirement, ResolutionGraph, Resolver,
 };
-use uv_toolchain::PythonEnvironment;
 use uv_types::{HashStrategy, InFlight, InstalledPackagesProvider};
 use uv_warnings::warn_user;
 
@@ -180,7 +180,11 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     .await?;
 
     // Collect constraints and overrides.
-    let constraints = Constraints::from_requirements(constraints);
+    let constraints = Constraints::from_requirements(
+        constraints
+            .into_iter()
+            .chain(upgrade.constraints().cloned()),
+    );
     let overrides = Overrides::from_requirements(overrides);
     let preferences = Preferences::from_iter(preferences, markers);
 
@@ -251,9 +255,9 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
         printer.stderr(),
         "{}",
         format!(
-            "Resolved {} in {}",
+            "Resolved {} {}",
             format!("{} package{}", resolution.len(), s).bold(),
-            elapsed(start.elapsed())
+            format!("in {}", elapsed(start.elapsed())).dimmed()
         )
         .dimmed()
     )?;
@@ -342,9 +346,9 @@ pub(crate) async fn install(
             printer.stderr(),
             "{}",
             format!(
-                "Audited {} in {}",
+                "Audited {} {}",
                 format!("{} package{}", resolution.len(), s).bold(),
-                elapsed(start.elapsed())
+                format!("in {}", elapsed(start.elapsed())).dimmed()
             )
             .dimmed()
         )?;
@@ -386,9 +390,9 @@ pub(crate) async fn install(
             printer.stderr(),
             "{}",
             format!(
-                "Prepared {} in {}",
+                "Prepared {} {}",
                 format!("{} package{}", wheels.len(), s).bold(),
-                elapsed(start.elapsed())
+                format!("in {}", elapsed(start.elapsed())).dimmed()
             )
             .dimmed()
         )?;
@@ -433,31 +437,34 @@ pub(crate) async fn install(
             printer.stderr(),
             "{}",
             format!(
-                "Uninstalled {} in {}",
+                "Uninstalled {} {}",
                 format!("{} package{}", extraneous.len() + reinstalls.len(), s).bold(),
-                elapsed(start.elapsed())
+                format!("in {}", elapsed(start.elapsed())).dimmed()
             )
             .dimmed()
         )?;
     }
 
     // Install the resolved distributions.
-    let wheels = wheels.into_iter().chain(cached).collect::<Vec<_>>();
+    let mut wheels = wheels.into_iter().chain(cached).collect::<Vec<_>>();
     if !wheels.is_empty() {
         let start = std::time::Instant::now();
-        uv_installer::Installer::new(venv)
+        wheels = uv_installer::Installer::new(venv)
             .with_link_mode(link_mode)
             .with_reporter(InstallReporter::from(printer).with_length(wheels.len() as u64))
-            .install(&wheels)?;
+            // This technically can block the runtime, but we are on the main thread and
+            // have no other running tasks at this point, so this lets us avoid spawning a blocking
+            // task.
+            .install_blocking(wheels)?;
 
         let s = if wheels.len() == 1 { "" } else { "s" };
         writeln!(
             printer.stderr(),
             "{}",
             format!(
-                "Installed {} in {}",
+                "Installed {} {}",
                 format!("{} package{}", wheels.len(), s).bold(),
-                elapsed(start.elapsed())
+                format!("in {}", elapsed(start.elapsed())).dimmed()
             )
             .dimmed()
         )?;
@@ -501,9 +508,9 @@ fn report_dry_run(
             printer.stderr(),
             "{}",
             format!(
-                "Audited {} in {}",
+                "Audited {} {}",
                 format!("{} package{}", resolution.len(), s).bold(),
-                elapsed(start.elapsed())
+                format!("in {}", elapsed(start.elapsed())).dimmed()
             )
             .dimmed()
         )?;
