@@ -11,9 +11,10 @@ use uv_python::{PythonFetch, PythonPreference, PythonRequest};
 use uv_warnings::{warn_user, warn_user_once};
 
 use crate::commands::pip::operations::Modifications;
+use crate::commands::project::lock::commit;
 use crate::commands::{project, ExitStatus, SharedState};
 use crate::printer::Printer;
-use crate::settings::{InstallerSettings, ResolverSettings};
+use crate::settings::ResolverInstallerSettings;
 
 /// Remove one or more packages from the project requirements.
 pub(crate) async fn remove(
@@ -21,6 +22,7 @@ pub(crate) async fn remove(
     dependency_type: DependencyType,
     package: Option<PackageName>,
     python: Option<String>,
+    settings: ResolverInstallerSettings,
     python_preference: PythonPreference,
     python_fetch: PythonFetch,
     preview: PreviewMode,
@@ -94,17 +96,18 @@ pub(crate) async fn remove(
     )
     .await?;
 
-    // Use the default settings.
-    let settings = ResolverSettings::default();
-
     // Initialize any shared state.
     let state = SharedState::default();
 
-    // Lock and sync the environment.
+    // Read the existing lockfile.
+    let existing = project::lock::read(project.workspace()).await?;
+
+    // Lock and sync the environment, if necessary.
     let lock = project::lock::do_lock(
         project.workspace(),
         venv.interpreter(),
-        settings.as_ref(),
+        existing.as_ref(),
+        settings.as_ref().into(),
         &state,
         preview,
         connectivity,
@@ -114,10 +117,12 @@ pub(crate) async fn remove(
         printer,
     )
     .await?;
+    if !existing.is_some_and(|existing| existing == lock) {
+        commit(&lock, project.workspace()).await?;
+    }
 
     // Perform a full sync, because we don't know what exactly is affected by the removal.
     // TODO(ibraheem): Should we accept CLI overrides for this? Should we even sync here?
-    let settings = InstallerSettings::default();
     let extras = ExtrasSpecification::All;
     let dev = true;
 
@@ -128,7 +133,7 @@ pub(crate) async fn remove(
         extras,
         dev,
         Modifications::Exact,
-        settings.as_ref(),
+        settings.as_ref().into(),
         &state,
         preview,
         connectivity,
