@@ -24,6 +24,7 @@ use uv_tool::InstalledTools;
 use uv_warnings::warn_user_once;
 
 use crate::commands::project::environment::CachedEnvironment;
+use crate::commands::reporters::PythonDownloadReporter;
 use crate::commands::tool::common::resolve_requirements;
 use crate::commands::{ExitStatus, SharedState};
 use crate::printer::Printer;
@@ -154,6 +155,8 @@ async fn get_or_create_environment(
         .connectivity(connectivity)
         .native_tls(native_tls);
 
+    let reporter = PythonDownloadReporter::single(printer);
+
     let python_request = python.map(PythonRequest::parse);
 
     // Discover an interpreter.
@@ -164,6 +167,7 @@ async fn get_or_create_environment(
         python_fetch,
         &client_builder,
         cache,
+        Some(&reporter),
     )
     .await?
     .into_interpreter();
@@ -214,7 +218,8 @@ async fn get_or_create_environment(
 
     // Check if the tool is already installed in a compatible environment.
     if !isolated {
-        let installed_tools = InstalledTools::from_settings()?;
+        let installed_tools = InstalledTools::from_settings()?.init()?;
+        let _lock = installed_tools.acquire_lock()?;
 
         let existing_environment =
             installed_tools
@@ -284,7 +289,7 @@ fn parse_target(target: &OsString) -> Result<(Cow<OsString>, Cow<str>)> {
 
     // e.g. ignore `git+https://github.com/uv/uv.git@main`
     if PackageName::from_str(name).is_err() {
-        debug!("Ignoring non-package name `{}` in command", name);
+        debug!("Ignoring non-package name `{name}` in command");
         return Ok((Cow::Borrowed(target), Cow::Borrowed(target_str)));
     }
 
@@ -297,6 +302,6 @@ fn parse_target(target: &OsString) -> Result<(Cow<OsString>, Cow<str>)> {
     }
 
     // e.g. `uv@invalid`, warn and treat the whole thing as the command
-    debug!("Ignoring invalid version request `{}` in command", version);
+    debug!("Ignoring invalid version request `{version}` in command");
     Ok((Cow::Borrowed(target), Cow::Borrowed(target_str)))
 }
