@@ -72,10 +72,10 @@ pub struct TestContext {
     /// The Python version used for the virtual environment, if any.
     pub python_version: Option<PythonVersion>,
 
-    // All the Python versions available during this test context.
+    /// All the Python versions available during this test context.
     pub python_versions: Vec<(PythonVersion, PathBuf)>,
 
-    // Standard filters for this test context
+    /// Standard filters for this test context.
     filters: Vec<(String, String)>,
 }
 
@@ -120,7 +120,33 @@ impl TestContext {
     #[must_use]
     pub fn with_filtered_exe_suffix(mut self) -> Self {
         self.filters
-            .push((std::env::consts::EXE_SUFFIX.to_string(), String::new()));
+            .push((regex::escape(env::consts::EXE_SUFFIX), String::new()));
+        self
+    }
+
+    /// Add extra standard filtering for Python executable names.
+    #[must_use]
+    pub fn with_filtered_python_names(mut self) -> Self {
+        if cfg!(windows) {
+            self.filters
+                .push(("python.exe".to_string(), "python".to_string()));
+        } else {
+            self.filters
+                .push((r"python\d".to_string(), "python".to_string()));
+            self.filters
+                .push((r"python\d.\d\d".to_string(), "python".to_string()));
+        }
+        self
+    }
+
+    /// Add extra standard filtering for venv executable directories on the current platform e.g.
+    /// `Scripts` on Windows and `bin` on Unix.
+    #[must_use]
+    pub fn with_filtered_virtualenv_bin(mut self) -> Self {
+        self.filters.push((
+            format!(r"[\\/]{}", venv_bin_path(PathBuf::new()).to_string_lossy()),
+            "/[BIN]".to_string(),
+        ));
         self
     }
 
@@ -207,7 +233,12 @@ impl TestContext {
             filters.extend(
                 Self::path_patterns(&python_dir.join(version.to_string()))
                     .into_iter()
-                    .map(|pattern| (format!("{pattern}.*"), format!("[PYTHON-{version}]"))),
+                    .map(|pattern| {
+                        (
+                            format!("{pattern}[a-zA-Z0-9]*"),
+                            format!("[PYTHON-{version}]"),
+                        )
+                    }),
             );
 
             // Add Python patch version filtering unless explicitly requested to ensure
@@ -281,6 +312,13 @@ impl TestContext {
         }
     }
 
+    /// Create a `uv` command for testing.
+    pub fn command(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        self.add_shared_args(&mut command);
+        command
+    }
+
     /// Shared behaviour for almost all test commands.
     ///
     /// * Use a temporary cache directory
@@ -306,7 +344,7 @@ impl TestContext {
         if cfg!(all(windows, debug_assertions)) {
             // TODO(konstin): Reduce stack usage in debug mode enough that the tests pass with the
             // default windows stack of 1MB
-            command.env("UV_STACK_SIZE", (8 * 1024 * 1024).to_string());
+            command.env("UV_STACK_SIZE", (2 * 1024 * 1024).to_string());
         }
     }
 
@@ -359,6 +397,14 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv help` command with options shared across scenarios.
+    #[allow(clippy::unused_self)]
+    pub fn help(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("help");
+        command
+    }
+
     /// Create a `uv sync` command with options shared across scenarios.
     pub fn sync(&self) -> Command {
         let mut command = Command::new(get_bin());
@@ -388,6 +434,19 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv python pin` command with options shared across scenarios.
+    pub fn python_pin(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command
+            .arg("python")
+            .arg("pin")
+            .env("UV_PREVIEW", "1")
+            .env("UV_PYTHON_INSTALL_DIR", "")
+            .current_dir(&self.temp_dir);
+        self.add_shared_args(&mut command);
+        command
+    }
+
     /// Create a `uv python dir` command with options shared across scenarios.
     pub fn python_dir(&self) -> Command {
         let mut command = Command::new(get_bin());
@@ -472,6 +531,14 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv tree` command with options shared across scenarios.
+    pub fn tree(&self) -> Command {
+        let mut command = Command::new(get_bin());
+        command.arg("tree");
+        self.add_shared_args(&mut command);
+        command
+    }
+
     /// Create a `uv clean` command.
     pub fn clean(&self) -> Command {
         let mut command = Command::new(get_bin());
